@@ -2,4 +2,4 @@ from __future__ import absolute_import
 # Copyright (c) 2010-2016 openpyxl
 
 
-from .comments import *
+from .comments import Comment
