@@ -5,17 +5,16 @@ from __future__ import absolute_import
 
 
 # Python stdlib imports
-from itertools import islice, chain
+from itertools import islice, product
 import re
 from inspect import isgenerator
-from weakref import ref
+from warnings import warn
 
 # compatibility imports
 from openpyxl.compat import (
     unicode,
     range,
     basestring,
-    iteritems,
     deprecated,
     safe_string
 )
@@ -23,14 +22,15 @@ from openpyxl.compat import (
 # package imports
 from openpyxl.utils import (
     coordinate_from_string,
-    COORD_RE,
-    ABSOLUTE_RE,
     column_index_from_string,
     get_column_letter,
     range_boundaries,
     rows_from_range,
     coordinate_to_tuple,
+    absolute_coordinate,
 )
+from openpyxl.utils.cell import COORD_RE
+
 from openpyxl.cell import Cell
 from openpyxl.utils.exceptions import (
     SheetTitleException,
@@ -40,23 +40,39 @@ from openpyxl.utils.exceptions import (
 from openpyxl.utils.units import (
     points_to_pixels,
     DEFAULT_COLUMN_WIDTH,
-    DEFAULT_ROW_HEIGHT
+    DEFAULT_ROW_HEIGHT,
 )
-from openpyxl.formatting import ConditionalFormatting
-from openpyxl.workbook.names.named_range import NamedRange
+from openpyxl.formatting.formatting import ConditionalFormattingList
+from openpyxl.packaging.relationship import RelationshipList
 from openpyxl.workbook.child import _WorkbookChild
+from openpyxl.workbook.defined_name import COL_RANGE_RE, ROW_RANGE_RE
 from openpyxl.utils.bound_dictionary import BoundDictionary
 
-from .header_footer import HeaderFooter
-from .page import PrintPageSetup, PageMargins, PrintOptions
-from .dimensions import ColumnDimension, RowDimension, DimensionHolder
+from .datavalidation import DataValidationList
+from .page import (
+    PrintPageSetup,
+    PageMargins,
+    PrintOptions,
+)
+from .dimensions import (
+    ColumnDimension,
+    RowDimension,
+    DimensionHolder,
+    SheetFormatProperties,
+)
 from .protection import SheetProtection
-from .filters import AutoFilter
-from .views import SheetView, Pane, Selection
+from .filters import AutoFilter, SortState
+from .views import (
+    SheetView,
+    Pane,
+    Selection,
+    SheetViewList,
+)
 from .properties import WorksheetProperties
 from .pagebreak import PageBreak
 
 
+@deprecated("Use the worksheet.values property")
 def flatten(results):
     """Return cell values row-by-row"""
 
@@ -72,6 +88,10 @@ class Worksheet(_WorkbookChild):
 
     """
 
+    _rel_type = "worksheet"
+    _path = "/xl/worksheets/sheet{0}.xml"
+    mime_type = "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml"
+
     BREAK_NONE = 0
     BREAK_ROW = 1
     BREAK_COLUMN = 2
@@ -98,7 +118,10 @@ class Worksheet(_WorkbookChild):
     ORIENTATION_LANDSCAPE = 'landscape'
 
     def __init__(self, parent, title=None):
-        super(Worksheet, self).__init__(parent, title)
+        _WorkbookChild.__init__(self, parent, title)
+        self._setup()
+
+    def _setup(self):
         self.row_dimensions = BoundDictionary("index", self._add_row)
         self.column_dimensions = DimensionHolder(worksheet=self,
                                                  default_factory=self._add_column)
@@ -106,35 +129,45 @@ class Worksheet(_WorkbookChild):
         self._cells = {}
         self._charts = []
         self._images = []
-        self._rels = []
+        self._rels = RelationshipList()
         self._drawing = None
-        self._comment_count = 0
+        self._comments = []
         self._merged_cells = []
-        self.hyperlinks = set()
-        self._data_validations = []
-        self.sheet_state = self.SHEETSTATE_VISIBLE
+        self._tables = []
+        self.data_validations = DataValidationList()
+        self._hyperlinks = []
+        self.sheet_state = 'visible'
         self.page_setup = PrintPageSetup(worksheet=self)
         self.print_options = PrintOptions()
+        self._print_rows = None
+        self._print_cols = None
+        self._print_area = None
         self.page_margins = PageMargins()
-        self.header_footer = HeaderFooter()
-        self.sheet_view = SheetView()
+        self.views = SheetViewList()
         self.protection = SheetProtection()
 
         self._current_row = 0
-        self._auto_filter = AutoFilter()
-        self._freeze_panes = None
+        self.auto_filter = AutoFilter()
+        self.sort_state = SortState()
         self.paper_size = None
         self.formula_attributes = {}
         self.orientation = None
-        self.conditional_formatting = ConditionalFormatting()
+        self.conditional_formatting = ConditionalFormattingList()
         self.legacy_drawing = None
         self.sheet_properties = WorksheetProperties()
+        self.sheet_format = SheetFormatProperties()
+
+
+    @property
+    def sheet_view(self):
+        return self.views.sheetView[0]
 
 
     @property
     def selected_cell(self):
         return self.sheet_view.selection.sqref
 
+
     @property
     def active_cell(self):
         return self.sheet_view.selection.activeCell
@@ -173,45 +206,20 @@ class Worksheet(_WorkbookChild):
     """ End To keep compatibility with previous versions"""
 
 
-    def _garbage_collect(self):
-        """Delete cells that are not storing a value."""
-        delete_list = []
-        for coordinate, cell in iteritems(self._cells):
-            if (cell.value in ('', None)
-            and cell.comment is None
-            and (cell.style_id == 0)):
-                delete_list.append(coordinate)
-        for coordinate in delete_list:
-            del self._cells[coordinate]
-
-
+    @deprecated("Use the ws.values property")
     def get_cell_collection(self):
         """Return an unordered list of the cells in this worksheet."""
         return self._cells.values()
 
 
-    @property
-    def auto_filter(self):
-        """Return :class:`~openpyxl.worksheet.AutoFilter` object.
-
-        `auto_filter` attribute stores/returns string until 1.8. You should change your code like ``ws.auto_filter.ref = "A1:A3"``.
-
-        .. versionchanged:: 1.9
-        """
-        return self._auto_filter
-
     @property
     def freeze_panes(self):
         if self.sheet_view.pane is not None:
             return self.sheet_view.pane.topLeftCell
 
     @freeze_panes.setter
-    def freeze_panes(self, topLeftCell):
-        if not topLeftCell:
-            topLeftCell = None
-        elif isinstance(topLeftCell, str):
-            topLeftCell = topLeftCell.upper()
-        else:  # Assume a cell
+    def freeze_panes(self, topLeftCell=None):
+        if isinstance(topLeftCell, Cell):
             topLeftCell = topLeftCell.coordinate
         if topLeftCell == 'A1':
             topLeftCell = None
@@ -220,9 +228,7 @@ class Worksheet(_WorkbookChild):
             self.sheet_view.pane = None
             return
 
-        if topLeftCell is not None:
-            colName, row = coordinate_from_string(topLeftCell)
-            column = column_index_from_string(colName)
+        row, column = coordinate_to_tuple(topLeftCell)
 
         view = self.sheet_view
         view.pane = Pane(topLeftCell=topLeftCell,
@@ -246,6 +252,8 @@ class Worksheet(_WorkbookChild):
             sel.insert(1, Selection(pane="bottomLeft", activeCell=None, sqref=None))
             view.selection = sel
 
+
+    @deprecated("Set print titles rows or columns directly")
     def add_print_title(self, n, rows_or_cols='rows'):
         """ Print Titles are rows or columns that are repeated on each printed sheet.
         This adds n rows or columns at the top or left of the sheet
@@ -254,25 +262,19 @@ class Worksheet(_WorkbookChild):
         scope = self.parent.get_index(self)
 
         if rows_or_cols == 'cols':
-            r = '$A:$%s' % get_column_letter(n)
+            self.print_title_cols = 'A:%s' % get_column_letter(n)
+
         else:
-            r = '$1:$%d' % n
+            self.print_title_rows = '1:%d' % n
 
-        self.parent.create_named_range('_xlnm.Print_Titles', self, r, scope)
 
     def cell(self, coordinate=None, row=None, column=None, value=None):
         """Returns a cell object based on the given coordinates.
 
-        Usage: cell(coodinate='A15') **or** cell(row=15, column=1)
+        Usage: cell(row=15, column=1, value=5)
 
-        If `coordinates` are not given, then row *and* column must be given.
-
-        Cells are kept in a dictionary which is empty at the worksheet
-        creation.  Calling `cell` creates the cell in memory when they
-        are first accessed, to reduce memory usage.
-
-        :param coordinate: coordinates of the cell (e.g. 'B12')
-        :type coordinate: string
+        Calling `cell` creates cells in memory when they
+        are first accessed.
 
         :param row: row index of the cell (e.g. 4)
         :type row: int
@@ -280,22 +282,26 @@ class Worksheet(_WorkbookChild):
         :param column: column index of the cell (e.g. 3)
         :type column: int
 
-        :raise: InsufficientCoordinatesException when coordinate or (row and column) are not given
+        :param coordinate: coordinates of the cell (e.g. 'B12')
+        :type coordinate: string
 
-        :rtype: :class:openpyxl.cell.Cell
+        :param value: value of the cell (e.g. 5)
+        :type value: numeric or time or string or bool or none
+
+        :raise: InsufficientCoordinatesException when neither row nor column are not given
+
+        :rtype: openpyxl.cell.Cell
 
         """
-        if coordinate is None:
-            if (row is None or column is None):
-                msg = "You have to provide a value either for " \
-                    "'coordinate' or for 'row' *and* 'column'"
-                raise InsufficientCoordinatesException(msg)
-            coordinate = (row, column)
 
-        else:
-            coordinate = coordinate.upper().replace('$', '')
-            coordinate = coordinate_to_tuple(coordinate)
-            row, column = coordinate
+        if (row is None or column is None) and coordinate is None:
+            msg = "You have to provide a value either for " \
+                   "'coordinate' or for 'row' *and* 'column'"
+            raise InsufficientCoordinatesException(msg)
+
+        if coordinate is not None:
+            warn("Using a coordinate with ws.cell is deprecated. Use ws[coordinate] instead")
+            row, column = coordinate_to_tuple(coordinate)
 
         if row < 1 or column < 1:
             raise ValueError("Row or column values must be at least 1")
@@ -330,13 +336,45 @@ class Worksheet(_WorkbookChild):
 
 
     def __getitem__(self, key):
-        """Convenience access by Excel style address"""
+        """Convenience access by Excel style coordinates
+
+        The key can be a single cell coordinate 'A1', a range of cells 'A1:D25',
+        individual rows or columns 'A', 4 or ranges of rows or columns 'A:D',
+        4:10.
+
+        Single cells will always be created if they do not exist.
+
+        Returns either a single cell or a tuple of rows or columns.
+        """
         if isinstance(key, slice):
-            return self.iter_rows("{0}:{1}".format(key.start, key.stop))
-        if ":" in key:
-            return self.iter_rows(key)
-        row, column = coordinate_to_tuple(key)
-        return self._get_cell(row, column)
+            if not all([key.start, key.stop]):
+                raise IndexError("{0} is not a valid coordinate or range".format(key))
+            key = "{0}:{1}".format(key.start, key.stop)
+
+        if isinstance(key, int):
+            key = str(key
+                      )
+        min_col, min_row, max_col, max_row = range_boundaries(key)
+
+        if not any([min_col, min_row, max_col, max_row]):
+            raise IndexError("{0} is not a valid coordinate or range".format(key))
+
+        if not min_row:
+            cols = tuple(self.iter_cols(min_col, max_col))
+            if min_col == max_col:
+                cols = cols[0]
+            return cols
+        if not min_col:
+            rows = tuple(self.iter_rows(min_col=min_col, min_row=min_row,
+                                        max_col=self.max_column, max_row=max_row))
+            if min_row == max_row:
+                rows = rows[0]
+            return rows
+        if ":" not in key:
+            return self._get_cell(min_row, min_col)
+        return tuple(self.iter_rows(min_row=min_row, min_col=min_col,
+                                    max_row=max_row, max_col=max_col))
+
 
     def __setitem__(self, key, value):
         self[key].value = value
@@ -346,11 +384,6 @@ class Worksheet(_WorkbookChild):
         return self.iter_rows()
 
 
-    @deprecated("Use the max_row property")
-    def get_highest_row(self):
-        return self.max_row
-
-
     @property
     def min_row(self):
         min_row = 1
@@ -373,11 +406,6 @@ class Worksheet(_WorkbookChild):
         return max_row
 
 
-    @deprecated("Use the max_column propery.")
-    def get_highest_column(self):
-        return self.max_column
-
-
     @property
     def min_column(self):
         min_col = 1
@@ -426,27 +454,55 @@ class Worksheet(_WorkbookChild):
         return self.calculate_dimension()
 
 
-    def iter_rows(self, range_string=None, row_offset=0, column_offset=0):
+    def iter_rows(self, range_string=None, min_row=None, max_row=None, min_col=None, max_col=None,
+                  row_offset=0, column_offset=0):
         """
-        Returns a squared range based on the `range_string` parameter,
-        using generators.
-        If no range is passed, will iterate over all cells in the worksheet
+        Return cells from the worksheet as rows. Boundaries for the cells can
+        be passed in either as indices of rows and columns.
 
-        :param range_string: range of cells (e.g. 'A1:C4')
+        If no boundaries are passed in the cells will start at A1.
+
+        If no cells are in the worksheet an empty tuple will be returned.
+
+
+        Additional rows and columns can be created using offsets.
+
+        :param range_string: range string (e.g. 'A1:B2') *deprecated*
         :type range_string: string
 
-        :param row_offset: additional rows (e.g. 4)
-        :type row: int
+        :param min_col: smallest column index (1-based index)
+        :type min_col: int
 
-        :param column_offset: additonal columns (e.g. 3)
-        :type column: int
+        :param min_row: smallest row index (1-based index)
+        :type min_row: int
+
+        :param max_col: largest column index (1-based index)
+        :type max_col: int
+
+        :param max_row: smallest row index (1-based index)
+        :type max_row: int
+
+        :param row_offset: additional rows (e.g. 4)
+        :type row_offset: int
+
+        :param column_offset: additional columns (e.g. 3)
+        :type column_offset: int
 
         :rtype: generator
         """
+
         if range_string is not None:
+            warn("Using a range string is deprecated. Use ws[range_string]")
             min_col, min_row, max_col, max_row = range_boundaries(range_string.upper())
-        else:
-            min_col, min_row, max_col, max_row = (1, 1, self.max_column, self.max_row)
+
+        if self._current_row == 0 and not any([min_col, min_row, max_col, max_row ]):
+            return ()
+
+        min_col = min_col or 1
+        min_row = min_row or 1
+        max_col = max_col or self.max_column
+        max_row = max_row or self.max_row
+
         if max_col is not None:
             max_col += column_offset
         if max_row is not None:
@@ -457,8 +513,28 @@ class Worksheet(_WorkbookChild):
                                       max_row)
 
 
-    def get_squared_range(self, min_col, min_row, max_col, max_row):
-        """Returns a 2D array of cells
+    @property
+    def rows(self):
+        """Iterate over all rows in the worksheet"""
+        return self.iter_rows()
+
+
+    @property
+    def values(self):
+        """
+        Return the values of all the cells, row by row
+        """
+        for row in self.iter_rows():
+            yield tuple(c.value for c in row)
+
+
+    def iter_cols(self, min_col=None, max_col=None, min_row=None, max_row=None):
+        """
+        Returns all cells in the worksheet from the first row as columns.
+
+        If no boundaries are passed in the cells will start at A1.
+
+        If no cells are in the worksheet an empty tuple will be returned.
 
         :param min_col: smallest column index (1-based index)
         :type min_col: int
@@ -474,73 +550,98 @@ class Worksheet(_WorkbookChild):
 
         :rtype: generator
         """
-        # Column name cache is very important in large files.
+
+        if self._current_row == 0 and not any([min_col, min_row, max_col, max_row ]):
+            return ()
+
+        min_col = min_col or 1
+        min_row = min_row or 1
+        max_col = max_col or self.max_column
+        max_row = max_row or self.max_row
+
+        return self._cells_by_col(
+            min_col, min_row, max_col, max_row
+        )
+
+
+    def _cells_by_col(self, min_col, min_row, max_col, max_row):
+        """
+        Get cells by column
+        """
+        for column in range(min_col, max_col+1):
+            yield tuple(self.cell(row=row, column=column)
+                        for row in range(min_row, max_row+1))
+
+
+    @property
+    def columns(self):
+        """Iterate over all columns in the worksheet"""
+        return self.iter_cols()
+
+
+    @deprecated("""
+    Use ws.iter_rows() or ws.iter_cols() depending whether you
+    want rows or columns returned.
+    """)
+    def get_squared_range(self, min_col, min_row, max_col, max_row):
+        """Returns a 2D array of cells. Will create any cells within the
+        boundaries that do not already exist
+
+        :param min_col: smallest column index (1-based index)
+        :type min_col: int
+
+        :param min_row: smallest row index (1-based index)
+        :type min_row: int
+
+        :param max_col: largest column index (1-based index)
+        :type max_col: int
+
+        :param max_row: smallest row index (1-based index)
+        :type max_row: int
+
+        :rtype: generator
+        """
+
         for row in range(min_row, max_row + 1):
             yield tuple(self.cell(row=row, column=column)
                         for column in range(min_col, max_col + 1))
 
 
-    def get_named_range(self, range_string):
+    @deprecated("""Ranges are workbook objects. Use wb.defined_names[range_name]""")
+    def get_named_range(self, range_name):
         """
         Returns a 2D array of cells, with optional row and column offsets.
 
-        :param range_string: `named range` name
-        :type range_string: string
+        :param range_name: `named range` name
+        :type range_name: string
 
-        :rtype: tuples of tuples of :class:`openpyxl.cell.Cell`
+        :rtype: tuple[tuple[openpyxl.cell.Cell]]
         """
-        named_range = self.parent.get_named_range(range_string)
-        if named_range is None:
-            msg = '%s is not a valid range name' % range_string
-            raise NamedRangeException(msg)
-        if not isinstance(named_range, NamedRange):
-            msg = '%s refers to a value, not a range' % range_string
-            raise NamedRangeException(msg)
+        defn = self.parent.defined_names[range_name]
+        if defn.localSheetId and defn.localSheetId != self.parent.get_index(self):
+            msg = "{0} not available in this worksheet".format(range_name)
+            raise KeyError(msg)
+
+        if defn.type != "RANGE":
+            msg = '{0} refers to a value, not a range'.format(range_name)
+            raise NameError(msg)
 
         result = []
-        for destination in named_range.destinations:
-            worksheet, cells_range = destination
+        for title, cells_range in defn.destinations:
+            ws = self.parent[title]
+            if ws != self:
+                raise NamedRangeException("Range includes cells from another worksheet")
 
-            if worksheet is not self:
-                msg = 'Range %s is not defined on worksheet %s' % \
-                    (cells_range, self.title)
-                raise NamedRangeException(msg)
+            rows = ws[cells_range]
+            if isinstance(rows, Cell):
+                rows = [(rows, )]
 
-            for row in self.iter_rows(cells_range):
+            for row in rows:
                 result.extend(row)
 
         return tuple(result)
 
 
-    @deprecated("""
-    Use .iter_rows() working with coordinates 'A1:D4',
-    and .get_squared_range() when working with indices (1, 1, 4, 4)
-    and .get_named_range() for named ranges""")
-    def range(self, range_string, row=0, column=0):
-        """Returns a 2D array of cells, with optional row and column offsets.
-
-        :param range_string: cell range string or `named range` name
-        :type range_string: string
-
-        :param row: number of rows to offset
-        :type row: int
-
-        :param column: number of columns to offset
-        :type column: int
-
-        :rtype: tuples of tuples of :class:`openpyxl.cell.Cell`
-
-        """
-        _rs = range_string.upper()
-        m = ABSOLUTE_RE.match(_rs)
-         # R1C1 range
-        if m is not None:
-            rows = self.iter_rows(_rs, row_offset=row, column_offset=column)
-            return tuple(row for row in rows)
-        else:
-            return self.get_named_range(range_string)
-
-
     def set_printer_settings(self, paper_size, orientation):
         """Set printer settings """
 
@@ -555,8 +656,7 @@ class Worksheet(_WorkbookChild):
             object defines the type of data-validation to be applied and the
             cell or range of cells it should apply to.
         """
-        data_validation._sheet = self
-        self._data_validations.append(data_validation)
+        self.data_validations.append(data_validation)
 
     def add_chart(self, chart, anchor=None):
         """
@@ -566,7 +666,6 @@ class Worksheet(_WorkbookChild):
         if anchor is not None:
             chart.anchor = anchor
         self._charts.append(chart)
-        self.parent._charts.append(ref(chart))
 
     def add_image(self, img, anchor=None):
         """
@@ -577,42 +676,41 @@ class Worksheet(_WorkbookChild):
             cell = self[anchor]
             img.anchor(cell, anchortype="oneCell")
         self._images.append(img)
-        self.parent._images.append(ref(img))
+
+
+    def add_table(self, table):
+        self._tables.append(table)
 
 
     def merge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
         """ Set merge on a cell range.  Range is a cell range (e.g. A1:E1) """
-        if not range_string:
-            if (start_row is None
-                or start_column is None
-                or end_row is None
-                or end_column is None):
-                msg = "You have to provide a value either for "\
-                    "'coordinate' or for 'start_row', 'start_column', 'end_row' *and* 'end_column'"
-                raise InsufficientCoordinatesException(msg)
-            else:
-                range_string = '%s%s:%s%s' % (get_column_letter(start_column),
-                                              start_row,
-                                              get_column_letter(end_column),
-                                              end_row)
+        if not range_string and not all((start_row, start_column, end_row, end_column)):
+            msg = "You have to provide a value either for 'coordinate' or for\
+            'start_row', 'start_column', 'end_row' *and* 'end_column'"
+            raise ValueError(msg)
+        elif not range_string:
+            range_string = '%s%s:%s%s' % (get_column_letter(start_column),
+                                          start_row,
+                                          get_column_letter(end_column),
+                                          end_row)
         elif ":" not in range_string:
             if COORD_RE.match(range_string):
-                return  # Single cell
-            msg = "Range must be a cell range (e.g. A1:E1)"
-            raise InsufficientCoordinatesException(msg)
+                return  # Single cell, do nothing
+            raise ValueError("Range must be a cell range (e.g. A1:E1)")
         else:
             range_string = range_string.replace('$', '')
 
         if range_string not in self._merged_cells:
             self._merged_cells.append(range_string)
 
-        cells = rows_from_range(range_string)
-        # only the top-left cell is preserved
-        for c in islice(chain.from_iterable(cells), 1, None):
+        min_col, min_row, max_col, max_row = range_boundaries(range_string)
+        rows = range(min_row, max_row+1)
+        cols = range(min_col, max_col+1)
+        cells = product(rows, cols)
+        # all but the top-left cell are removed
+        for c in islice(cells, 1, None):
             if c in self._cells:
                 del self._cells[c]
-            if c in self.hyperlinks:
-                del self._hyperlinks[c]
 
 
     @property
@@ -627,8 +725,8 @@ class Worksheet(_WorkbookChild):
 
     @property
     def merged_cell_ranges(self):
-        """Public attribute for which cells have been merged"""
-        return self._merged_cells
+        """Return a copy of cell ranges"""
+        return self._merged_cells[:]
 
 
     def unmerge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
@@ -660,7 +758,7 @@ class Worksheet(_WorkbookChild):
         * If it's a dict: values are assigned to the columns indicated by the keys (numbers or letters)
 
         :param iterable: list, range or generator, or dict containing values to append
-        :type iterable: list/tuple/range/generator or dict
+        :type iterable: list|tuple|range|generator or dict
 
         Usage:
 
@@ -679,8 +777,8 @@ class Worksheet(_WorkbookChild):
                 if isinstance(content, Cell):
                     # compatible with write-only mode
                     cell = content
-                    if cell.parent and cell.parent.parent != self.parent:
-                        raise ValueError("Cells cannot be copied from other workbooks")
+                    if cell.parent and cell.parent != self:
+                        raise ValueError("Cells cannot be copied from other worksheets")
                     cell.parent = self
                     cell.col_idx = col_idx
                     cell.row = row_idx
@@ -689,7 +787,7 @@ class Worksheet(_WorkbookChild):
                 self._cells[(row_idx, col_idx)] = cell
 
         elif isinstance(iterable, dict):
-            for col_idx, content in iteritems(iterable):
+            for col_idx, content in iterable.items():
                 if isinstance(col_idx, basestring):
                     col_idx = column_index_from_string(col_idx)
                 cell = Cell(self, row=row_idx, col_idx=col_idx, value=content)
@@ -706,30 +804,16 @@ class Worksheet(_WorkbookChild):
             type(iterable))
                         )
 
-    @property
-    def rows(self):
-        """Iterate over all rows in the worksheet"""
-        if self.min_row == self.max_row == self.min_column == self.max_column:
-            return ((),)
-        return tuple(self.iter_rows())
-
-
-    @property
-    def columns(self):
-        """Iterate over all columns in the worksheet"""
-        if self.min_row == self.max_row == self.min_column == self.max_column:
-            return ((),)
-        cols = []
-        for col_idx in range(self.max_column):
-            cells = self.get_squared_range(col_idx + 1, self.min_row, col_idx + 1, self.max_row)
-            col = chain.from_iterable(cells)
-            cols.append(tuple(col))
-        return tuple(cols)
 
+    @deprecated("Charts and images should be positioned using anchor objects")
     def point_pos(self, left=0, top=0):
         """ tells which cell is under the given coordinates (in pixels)
         counting from the top-left corner of the sheet.
         Can be used to locate images and charts on the worksheet """
+
+        if left < 0 or top < 0:
+            raise ValueError("Coordinates must be positive")
+
         current_col = 1
         current_row = 1
         column_dimensions = self.column_dimensions
@@ -773,6 +857,87 @@ class Worksheet(_WorkbookChild):
         return RowDimension(self)
 
 
-    def _write(self, shared_strings=None):
+    def _write(self):
+        from openpyxl.drawing.spreadsheet_drawing import SpreadsheetDrawing
         from openpyxl.writer.worksheet import write_worksheet
-        return write_worksheet(self, shared_strings)
+        self._drawing = SpreadsheetDrawing()
+        self._drawing.charts = self._charts
+        self._drawing.images = self._images
+        return write_worksheet(self)
+
+
+    @property
+    def print_titles(self):
+        """
+        Return the print titles for the worksheet as rows and columns,
+        if set.
+        """
+        if self.print_title_rows and self.print_title_cols:
+            return ",".join([self.print_title_rows, self.print_title_cols])
+        elif self.print_title_rows:
+            return self.print_title_rows
+        elif self.print_title_cols:
+            return self.print_title_cols
+
+
+    @property
+    def print_title_rows(self):
+        if self._print_rows:
+            return u"{0}!{1}".format(self.title, self._print_rows)
+
+
+    @print_title_rows.setter
+    def print_title_rows(self, rows):
+        """
+        Set rows to be printed on the top of every page
+        format `1:3`
+        """
+        if rows is not None:
+            if not ROW_RANGE_RE.match(rows):
+                raise ValueError("Print title rows must be the form 1:3")
+        self._print_rows = rows
+
+
+    @property
+    def print_title_cols(self):
+        if self._print_cols:
+            return u"{0}!{1}".format(self.title, self._print_cols)
+
+
+    @print_title_cols.setter
+    def print_title_cols(self, cols):
+        """
+        Set cols to be printed on the left of every page
+        format ``A:C`
+        """
+        if cols is not None:
+            if not COL_RANGE_RE.match(cols):
+                raise ValueError("Print title cols must be the form C:D")
+        self._print_cols = cols
+
+
+    @property
+    def print_titles(self):
+        if self.print_title_cols and self.print_title_rows:
+            return ",".join([self.print_title_rows, self.print_title_cols])
+        else:
+            return self.print_title_rows or self.print_title_cols
+
+
+    @property
+    def print_area(self):
+        """
+        Return the print area for the worksheet, if set
+        """
+        return self._print_area
+
+
+    @print_area.setter
+    def print_area(self, value):
+        """
+        Range of cells in the form A1:D4 or list of ranges
+        """
+        if isinstance(value, basestring):
+            value = [value]
+
+        self._print_area = [absolute_coordinate(v) for v in value]
